@@ -33,8 +33,10 @@
 #include "phonemgr-utils.h"
 
 /* #define DUMMY 1 */
-#define POLL_TIMEOUT 300
-#define TRYLOCK_TIMEOUT 50
+#define POLL_TIMEOUT 300 * 1000
+#define TRYLOCK_TIMEOUT 50 * 1000
+
+#define CHECK_EXIT { if (l->terminated != FALSE) { g_mutex_unlock (l->mutex); goto exit_thread; } }
 
 static gpointer		 parent_class = NULL;
 
@@ -69,6 +71,7 @@ struct _PhonemgrListener
 	PhonemgrState *phone_state;
 
 	char *driver;
+	char *own_number;
 
 	/* The previous call status */
 	gn_call_status prev_call_status;
@@ -98,6 +101,7 @@ static void phonemgr_listener_thread (PhonemgrListener *l);
 
 enum {
 	MESSAGE_SIGNAL,
+	REPORT_STATUS_SIGNAL,
 	STATUS_SIGNAL,
 	CALL_STATUS_SIGNAL,
 	BATTERY_SIGNAL,
@@ -173,6 +177,17 @@ phonemgr_listener_class_init (PhonemgrListenerClass *klass)
 			      3,
 			      G_TYPE_STRING, G_TYPE_ULONG, G_TYPE_STRING);
 
+	phonemgr_listener_signals[REPORT_STATUS_SIGNAL] =
+		g_signal_new ("report-status",
+			      G_OBJECT_CLASS_TYPE (klass),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (PhonemgrListenerClass, report_status),
+			      NULL, NULL,
+			      phonemgr_marshal_VOID__STRING_ULONG_INT,
+			      G_TYPE_NONE,
+			      3,
+			      G_TYPE_STRING, G_TYPE_ULONG, G_TYPE_INT);
+
 	phonemgr_listener_signals[STATUS_SIGNAL] =
 		g_signal_new ("status",
 			      G_OBJECT_CLASS_TYPE (klass),
@@ -234,6 +249,40 @@ phonemgr_listener_emit_status (PhonemgrListener *l, PhonemgrListenerStatus statu
 		       0, status);
 }
 
+static void
+phonemgr_listener_emit_report_status (PhonemgrListener *l, gn_sms *message)
+{
+	time_t time;
+	char *sender;
+	PhonemgrListenerReportStatus status;
+
+	switch (message->user_data[0].dr_status) {
+	case GN_SMS_DR_Status_Delivered:
+		status = PHONEMGR_LISTENER_REPORT_DELIVERED;
+		break;
+	case GN_SMS_DR_Status_Pending:
+		status = PHONEMGR_LISTENER_REPORT_PENDING;
+		break;
+	case GN_SMS_DR_Status_Failed_Temporary:
+		status = PHONEMGR_LISTENER_REPORT_FAILED_TEMPORARY;
+		break;
+	case GN_SMS_DR_Status_Failed_Permanent:
+		status = PHONEMGR_LISTENER_REPORT_FAILED_PERMANENT;
+		break;
+	default:
+		return;
+	}
+
+	time = gn_timestamp_to_gtime (message->smsc_time);
+	sender = g_strdup (message->remote.number);
+
+	g_signal_emit (G_OBJECT (l),
+		       phonemgr_listener_signals[REPORT_STATUS_SIGNAL],
+		       0, sender, time, status);
+
+	g_free (sender);
+}
+
 static void
 phonemgr_listener_emit_call_status (PhonemgrListener *l,
 				    PhonemgrListenerCallStatus status,
@@ -321,6 +370,7 @@ phonemgr_listener_init (PhonemgrListener *l)
 	l->queue = g_async_queue_new ();
 	l->mutex = g_mutex_new ();
 	l->driver = NULL;
+	l->own_number = NULL;
 	l->batterylevel = 1;
 	l->supports_power_source = TRUE;
 	l->powersource = GN_PS_BATTERY;
@@ -348,36 +398,55 @@ phonemgr_listener_connect (PhonemgrListener *l, char *device, GError **error)
 {
 	int channel;
 
+	g_return_val_if_fail (PHONEMGR_IS_LISTENER (l), FALSE);
 	g_return_val_if_fail (l->connected == FALSE, FALSE);
 	g_return_val_if_fail (l->phone_state == NULL, FALSE);
 
 	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_CONNECTING);
 
-
 	channel = -1;
 
-	if (phonemgr_utils_is_bluetooth (device) != FALSE) {
-		channel = phonemgr_utils_get_channel (device);
+	if (phonemgr_utils_address_is (device) == PHONEMGR_CONNECTION_BLUETOOTH) {
+		channel = phonemgr_utils_get_serial_channel (device);
 		if (channel < 0) {
 			//FIXME
 			return FALSE;
 		}
 	}
 
+	phonemgr_utils_free (l->phone_state);
 	l->phone_state = phonemgr_utils_connect (device, NULL, channel, l->debug, error);
 	if (l->phone_state == NULL) {
 		//FIXME
 		return FALSE;
 	}
 
+	g_free (l->driver);
 	l->driver = phonemgr_utils_guess_driver (l->phone_state, device, error);
 	if (l->driver == NULL) {
 		//FIXME
+		phonemgr_utils_disconnect (l->phone_state);
+		phonemgr_utils_free (l->phone_state);
 		return FALSE;
 	}
 
+	/* Do we need to see gnapplet driver? */
+	if (strcmp (l->driver, PHONEMGR_DEFAULT_GNAPPLET_DRIVER) == 0) {
+		channel = phonemgr_utils_get_gnapplet_channel (device);
+		if (channel < 0) {
+			phonemgr_utils_disconnect (l->phone_state);
+			phonemgr_utils_free (l->phone_state);
+			g_free (l->driver);
+			l->driver = NULL;
+			//FIXME
+			g_message ("Couldn't connect to the device, gnapplet needed but not running");
+			return FALSE;
+		}
+	}
+
 	/* Need a different driver? then reconnect */
-	if (strcmp (l->driver, PHONEMGR_DEFAULT_DRIVER) != 0) {
+	if (strcmp (l->driver, PHONEMGR_DEFAULT_DRIVER) != 0
+	    && strcmp (l->driver, PHONEMGR_DEFAULT_USB_DRIVER) != 0) {
 		phonemgr_utils_disconnect (l->phone_state);
 		phonemgr_utils_free (l->phone_state);
 		l->phone_state = phonemgr_utils_connect (device, l->driver, channel, l->debug, error);
@@ -393,7 +462,7 @@ phonemgr_listener_connect (PhonemgrListener *l, char *device, GError **error)
 	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_CONNECTED);
 
 	l->thread = g_thread_create ((GThreadFunc) phonemgr_listener_thread,
-			l, TRUE, NULL);
+				     l, TRUE, NULL);
 
 	return l->connected;
 }
@@ -409,11 +478,18 @@ phonemgr_listener_push (PhonemgrListener *l)
 	if (signal == NULL)
 		return FALSE;
 
-	if (signal->type == MESSAGE_SIGNAL) {
+	switch (signal->type) {
+	case MESSAGE_SIGNAL:
 		g_message ("emitting message");
 		phonemgr_listener_emit_message (l, signal->message);
 		g_free (signal->message);
-	} else if (signal->type == CALL_STATUS_SIGNAL) {
+		break;
+	case REPORT_STATUS_SIGNAL:
+		g_message ("emitting delivery report status");
+		phonemgr_listener_emit_report_status (l, signal->message);
+		g_free (signal->message);
+		break;
+	case CALL_STATUS_SIGNAL:
 		g_message ("emitting call status");
 		phonemgr_listener_emit_call_status (l, signal->call->status,
 						    signal->call->number,
@@ -421,13 +497,15 @@ phonemgr_listener_push (PhonemgrListener *l)
 		g_free (signal->call->number);
 		g_free (signal->call->name);
 		g_free (signal->call);
-	} else if (signal->type == BATTERY_SIGNAL) {
+		break;
+	case BATTERY_SIGNAL:
 		g_message ("emitting battery");
 		phonemgr_listener_emit_battery (l,
 						(int) signal->battery->batterylevel,
 						signal->battery->powersource != GN_PS_BATTERY);
 		g_free (signal->battery);
-	} else {
+		break;
+	default:
 		g_assert_not_reached ();
 	}
 
@@ -441,9 +519,18 @@ phonemgr_listener_new_sms_cb (gn_sms *message, struct gn_statemachine *state, vo
 {
 	PhonemgrListener *l = (PhonemgrListener *) user_data;
 	AsyncSignal *signal;
+	int type;
+
+	/* Check that it's a type of message we support */
+	if (message->type == GN_SMS_MT_DeliveryReport)
+		type = REPORT_STATUS_SIGNAL;
+	else if (message->type == GN_SMS_MT_Deliver)
+		type = MESSAGE_SIGNAL;
+	else
+		return;
 
 	signal = g_new0 (AsyncSignal, 1);
-	signal->type = MESSAGE_SIGNAL;
+	signal->type = type;
 	/* The message is allocated on the stack in the driver, so copy it */
 	signal->message = g_memdup (message, sizeof (gn_sms));
 
@@ -519,7 +606,7 @@ phonemgr_listener_new_call_cb (gn_call_status call_status, gn_call_info *call_in
 static void
 phonemgr_listener_sms_notification_soft_poll (PhonemgrListener *l)
 {
-	gn_sm_loop (1, &l->phone_state->state);
+	gn_sm_loop (10, &l->phone_state->state);
 	/* Some phones may not be able to notify us, thus we give
 	 * lowlevel chance to poll them */
 	gn_sm_functions (GN_OP_PollSMS, &l->phone_state->data, &l->phone_state->state);
@@ -579,9 +666,10 @@ phonemgr_listener_battery_poll (PhonemgrListener *l)
 		l->batterylevel = batterylevel;
 		l->powersource = powersource;
 
-		/* Probably not the best guess, but that's what xgnokii uses */
+		/* As mentioned by Daniele Forsi at:
+		 * http://thread.gmane.org/gmane.linux.drivers.gnokii/9329/focus=9331 */
 		if (battery_unit == GN_BU_Arbitrary)
-			batterylevel *= 25;
+			batterylevel = batterylevel * 100 / l->phone_state->state.driver.phone.max_battery_level;
 
 		signal = g_new0 (AsyncSignal, 1);
 		signal->type = BATTERY_SIGNAL;
@@ -595,6 +683,69 @@ phonemgr_listener_battery_poll (PhonemgrListener *l)
 
 }
 
+static void
+phonemgr_listener_get_own_details (PhonemgrListener *l)
+{
+	gn_memory_status memstat;
+	gn_phonebook_entry entry;
+	int count, start_entry, end_entry, num_entries;
+	gn_error err;
+
+	start_entry = 1;
+	end_entry = num_entries = INT_MAX;
+
+	memstat.memory_type = gn_str2memory_type("ON");
+	l->phone_state->data.memory_status = &memstat;
+	err = gn_sm_functions(GN_OP_GetMemoryStatus, &l->phone_state->data, &l->phone_state->state);
+	if (err == GN_ERR_NONE) {
+		num_entries = memstat.used;
+		end_entry = memstat.used + memstat.free;
+	} else if (err == GN_ERR_INVALIDMEMORYTYPE) {
+		g_message ("Couldn't get our own phone number (no Own Number phonebook)");
+		return;
+	}
+
+	count = start_entry;
+	while (num_entries > 0 && count <= end_entry) {
+		gn_error error = GN_ERR_NONE;
+
+		memset(&entry, 0, sizeof(gn_phonebook_entry));
+		entry.memory_type = memstat.memory_type;
+		entry.location = count;
+
+		l->phone_state->data.phonebook_entry = &entry;
+		error = gn_sm_functions(GN_OP_ReadPhonebook, &l->phone_state->data, &l->phone_state->state);
+		if (error != GN_ERR_NONE && error != GN_ERR_EMPTYLOCATION)
+			break;
+		if (entry.empty != FALSE)
+			continue;
+		if (error == GN_ERR_NONE)
+			num_entries--;
+		count++;
+
+		if (!entry.subentries_count && entry.number) {
+			l->own_number = g_strdup (entry.number);
+		} else {
+			int i;
+
+			for (i = 0; i < entry.subentries_count; i++) {
+				if (entry.subentries[i].entry_type != GN_PHONEBOOK_ENTRY_Number)
+					continue;
+
+				l->own_number = g_strdup (entry.number);
+				break;
+			}
+		}
+
+		break;
+	}
+
+	if (l->own_number != NULL)
+		g_message ("Our own phone number is: %s", l->own_number);
+	else
+		g_message ("Couldn't get our own phone number");
+}
+
 static void
 phonemgr_listener_set_sms_notification (PhonemgrListener *l, gboolean state)
 {
@@ -641,15 +792,23 @@ static void
 phonemgr_listener_thread (PhonemgrListener *l)
 {
 	g_mutex_lock (l->mutex);
+	CHECK_EXIT;
+	phonemgr_listener_get_own_details (l);
+	CHECK_EXIT;
 	phonemgr_listener_set_sms_notification (l, TRUE);
+	CHECK_EXIT;
 	phonemgr_listener_set_call_notification (l, TRUE);
+	CHECK_EXIT;
 	g_mutex_unlock (l->mutex);
 
 	while (l->terminated == FALSE) {
 		if (g_mutex_trylock (l->mutex) != FALSE) {
+			CHECK_EXIT;
 			if (l->supports_sms_notif != FALSE)
 				phonemgr_listener_sms_notification_soft_poll (l);
+			CHECK_EXIT;
 			phonemgr_listener_call_notification_poll (l);
+			CHECK_EXIT;
 			phonemgr_listener_battery_poll (l);
 
 			g_mutex_unlock (l->mutex);
@@ -659,6 +818,7 @@ phonemgr_listener_thread (PhonemgrListener *l)
 		}
 	}
 
+exit_thread:
 	g_mutex_lock (l->mutex);
 	phonemgr_listener_set_sms_notification (l, FALSE);
 	phonemgr_listener_set_call_notification (l, FALSE);
@@ -670,6 +830,7 @@ phonemgr_listener_thread (PhonemgrListener *l)
 void
 phonemgr_listener_disconnect (PhonemgrListener *l)
 {
+	g_return_if_fail (PHONEMGR_IS_LISTENER (l));
 	g_return_if_fail (l->connected != FALSE);
 
 	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_DISCONNECTING);
@@ -679,6 +840,8 @@ phonemgr_listener_disconnect (PhonemgrListener *l)
 
 	g_free (l->driver);
 	l->driver = NULL;
+	g_free (l->own_number);
+	l->own_number = NULL;
 
 	l->connected = FALSE;
 	//FIXME more to kill?
@@ -692,24 +855,29 @@ phonemgr_listener_disconnect (PhonemgrListener *l)
 void
 phonemgr_listener_cancel_call (PhonemgrListener *l)
 {
+	g_return_if_fail (PHONEMGR_IS_LISTENER (l));
 	//FIXME
 }
 
 void
 phonemgr_listener_answer_call (PhonemgrListener *l)
 {
+	g_return_if_fail (PHONEMGR_IS_LISTENER (l));
 	//FIXME
 }
 
 void
 phonemgr_listener_queue_message (PhonemgrListener *l,
-		const char *number, const char *message)
+				 const char *number,
+				 const char *message,
+				 gboolean delivery_report)
 {
 	char *mstr, *iso;
 	GError *err = NULL;
 	gn_sms sms;
 	gn_error error;
 
+	g_return_if_fail (PHONEMGR_IS_LISTENER (l));
 	g_return_if_fail (l->connected != FALSE);
 	g_return_if_fail (number != NULL);
 	g_return_if_fail (message != NULL);
@@ -721,9 +889,7 @@ phonemgr_listener_queue_message (PhonemgrListener *l,
 
 	/* Default GSM alphabet is a subset of ISO8859-1,
 	 * so try that */
-	iso = g_convert (message, -1,
-			 "ISO8859-1", "UTF-8",
-			 NULL, NULL, &err);
+	iso = g_convert (message, -1, "ISO8859-1", "UTF-8", NULL, NULL, &err);
 	if (err != NULL) {
 		g_clear_error (&err);
 		g_free (iso);
@@ -766,6 +932,9 @@ phonemgr_listener_queue_message (PhonemgrListener *l,
 	}
 	g_free (l->phone_state->data.message_center);
 
+	/* Set whether to get a delivery report */
+	sms.delivery_report = delivery_report;
+
 	/* Set the message data */
 	sms.user_data[0].type = GN_SMS_DATA_Text;
 	g_strlcpy ((char *) sms.user_data[0].u.text, mstr, strlen (mstr) + 1);
@@ -794,6 +963,10 @@ phonemgr_listener_set_time (PhonemgrListener *l,
 	gn_error error;
 	PhoneMgrError perr;
 
+	g_return_if_fail (PHONEMGR_IS_LISTENER (l));
+	g_return_if_fail (l->connected == FALSE);
+	g_return_if_fail (l->phone_state == NULL);
+
 	t = localtime(&time);
 	date.year = t->tm_year;
 	date.month = t->tm_mon + 1;
@@ -819,41 +992,15 @@ phonemgr_listener_set_time (PhonemgrListener *l,
 }
 
 gboolean
-phonemgr_listener_connected (PhonemgrListener *listener)
+phonemgr_listener_connected (PhonemgrListener *l)
 {
-	return listener->connected;
+	g_return_val_if_fail (PHONEMGR_IS_LISTENER (l), FALSE);
+
+	return l->connected;
 }
 
 #else /* !DUMMY */
 
-gboolean
-phonemgr_listener_connect (PhonemgrListener *l, char *device, GError **err)
-{
-	g_message ("[DUMMY] connecting to %s", device);
-	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_CONNECTING);
-	g_usleep (G_USEC_PER_SEC * 2);
-	l->connected = TRUE;
-	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_CONNECTED);
-
-	return TRUE;
-}
-
-void
-phonemgr_listener_disconnect (PhonemgrListener *l)
-{
-	g_message ("[DUMMY] disconnecting");
-	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_DISCONNECTING);
-	g_usleep (G_USEC_PER_SEC * 2);
-	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_IDLE);
-}
-
-void
-phonemgr_listener_queue_message (PhonemgrListener *listener,
-        const char *number, const char *message)
-{
-	g_message ("[DUMMY] sending message to %s: %s", number, message);
-}
-
 /* We poll every 50 milliseconds, so this is the number of times
  * we need to go through poll() to wait for one second */
 #define POLL_SECOND 1000 / 50
@@ -864,11 +1011,26 @@ phonemgr_listener_poll (PhonemgrListener *l)
 	static int i = 0;
 	static int target = 5 * POLL_SECOND;
 
+	if (l->connected == FALSE)
+		return FALSE;
+
 	i++;
 	if (i == target) {
-		char *sender = "+01234 567-89";
+		char *sender;
 		GTimeVal time;
-		char *text = "This is my test, this is my supa test";
+		char *text;
+		GRand *rand;
+
+		rand = g_rand_new_with_seed (i);
+
+		if (g_rand_boolean (rand)) {
+			sender = "+09876 543-21";
+			text = "This is my other t\303\251st, this is NOT a supa test: http://live.gnome.org/PhoneManager";
+		} else {
+			sender = "+01234 567-89";
+			text = "This is my test, this is my supa test with i18n chars: \303\270\317\216\307\252 (http://live.gnome.org/PhoneManager)";
+		}
+		g_rand_free (rand);
 
 		g_get_current_time (&time);
 
@@ -882,6 +1044,40 @@ phonemgr_listener_poll (PhonemgrListener *l)
 	return TRUE;
 }
 
+gboolean
+phonemgr_listener_connect (PhonemgrListener *l, char *device, GError **err)
+{
+	g_message ("[DUMMY] connecting to %s", device);
+	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_CONNECTING);
+	g_usleep (G_USEC_PER_SEC * 2);
+	l->connected = TRUE;
+	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_CONNECTED);
+
+	g_timeout_add (50, (GSourceFunc) phonemgr_listener_poll, l);
+
+	return TRUE;
+}
+
+void
+phonemgr_listener_disconnect (PhonemgrListener *l)
+{
+	g_message ("[DUMMY] disconnecting");
+	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_DISCONNECTING);
+	l->connected = FALSE;
+	g_usleep (G_USEC_PER_SEC * 2);
+	phonemgr_listener_emit_status (l, PHONEMGR_LISTENER_IDLE);
+}
+
+void
+phonemgr_listener_queue_message (PhonemgrListener *listener,
+				 const char *number,
+				 const char *message,
+				 gboolean delivery_report)
+{
+	g_message ("[DUMMY] sending message to %s: %s (Delivery report: %s)",
+		   number, message, delivery_report ? "Yes" : "No");
+}
+
 gboolean
 phonemgr_listener_connected (PhonemgrListener *listener)
 {
@@ -891,7 +1087,6 @@ phonemgr_listener_connected (PhonemgrListener *listener)
 static void
 phonemgr_listener_init (PhonemgrListener *l)
 {
-	g_timeout_add (50, (GSourceFunc) phonemgr_listener_poll, l);
 }
 
 static void
